// Generated by CoffeeScript 2.0.2
(function() {
  // If we make it to this file, the user has a S3 CORS configuration, but has
  // not set an explicit CF headers policy, so we need to fill in the gaps with
  // smart defaults and a warning if we need to.

  // Case 1)
  // =========================
  // If the S3 CORS configuration is permissive, it will be tempting to not set
  // the CloudFront Header Caching field.  If there is something in the headers
  // field, don't do anything because we assume user knows what they are doing.
  // But otherwise, we need to whitelist a permissive set of headers.

  // Case 2)
  // ========================
  // If there is an S3 CORS configuration, but it is not set to the permissive
  // catch-all, we need to warn the user that the default CloudFront
  // Header behavior does not cache anything.
  var permissiveHeaders, printWarning,
    indexOf = [].indexOf;

  permissiveHeaders = ["Accept", "Accept-Charset", "Accept-Datetime", "Accept-Language", "Access-Control-Request-Headers", "Access-Control-Request-Method", "Authorization", "Host", "Origin", "Referer"];

  printWarning = function() {
    var msg;
    msg = "\"\n==============================================================\nWARNING: There is an S3 CORS configuration, but the corresponding\nCloudFormation configuration does not include an explicit header caching\npolicy.\n\nBy default, CloudFront doesn't consider headers when caching your objects in\nedge locations. If your origin returns two objects and they differ only by the\nvalues in the request headers, CloudFront caches only one version of the object.\n\nIf this is intentional, you can hide this warning by setting, within your\nenvironment, cache.headers to []\n==============================================================";
    return console.warn(msg);
  };

  module.exports = function(out, defs) {
    var aws, ref;
    ({aws} = out);
    if (aws.cors && typeof aws.cors === "string" && (ref = aws.cors, indexOf.call(defs.corsRuleDefault.enum, ref) >= 0)) {
      out.aws.cache.headers = permissiveHeaders;
    } else if (aws.cors) {
      printWarning();
    }
    return out;
  };

}).call(this);
