// Generated by CoffeeScript 2.0.2
(function() {
  var async, cat, clone, collect, deepEqual, empty, randomKey, sleep, where,
    indexOf = [].indexOf;

  ({async, sleep, collect, where, empty, cat, clone, deepEqual} = require("fairmont"));

  ({randomKey} = require("key-forge"));

  // TODO add configuration for cloudfront restrictions.
  module.exports = async(function*(config, cf) {
    var acm, bucketURL, fullyQualify, invalidate, needsInvalidation, regularlyQualify, root, set, sync, syncInvalidation;
    ({bucketURL, fullyQualify, regularlyQualify, root} = require("../url")(config));
    acm = require("./acm")(config, ((yield require("../../../aws")("us-east-1"))).acm);
    needsInvalidation = function({dlist, ulist}) {
      return !empty(cat(dlist, ulist));
    };
    // Invalidate distribution cache.  Because we are charged per path we specify,
    // it's cheapest to just invalidate everything unless nothing changed.
    invalidate = async(function*(distribution) {
      var Invalidation, createInvalidation, e;
      createInvalidation = async(function*() {
        return (yield cf.createInvalidation({
          DistributionId: distribution.Id,
          InvalidationBatch: {
            CallerReference: "Haiku" + randomKey(32),
            Paths: {
              Quantity: 1,
              Items: ["/*"]
            }
          }
        }));
      });
      try {
        return {Invalidation} = (yield createInvalidation());
      } catch (error) {
        e = error;
        console.error("Unexpected response while setting invalidation.", e);
        throw new Error();
      }
    });
    set = async(function*(name) {
      var applyDefaults, buildConfiguration, buildOrigins, buildSource, confirmDistributionConfig, createDistribution, current, deepSort, e, id, list, match, matches, pattern, setHeaderCacheConfiguration, setViewerCertificate, updateDistribution;
      buildSource = function(name) {
        return name + ".s3-website-" + config.aws.region + ".amazonaws.com";
      };
      setViewerCertificate = async(function*() {
        var cert, protocol, ssl;
        ({ssl, protocol} = config.aws.cache);
        if (ssl) {
          cert = (yield acm.fetch(config.aws.hostnames[0]));
          return {
            ACMCertificateArn: cert,
            SSLSupportMethod: 'sni-only',
            MinimumProtocolVersion: protocol || 'TLSv1.2_2018',
            Certificate: cert,
            CertificateSource: 'acm'
          };
        } else {
          return {
            CloudFrontDefaultCertificate: true,
            MinimumProtocolVersion: 'SSLv3',
            CertificateSource: 'cloudfront'
          };
        }
      });
      setHeaderCacheConfiguration = function() {
        var headers;
        ({headers} = config.aws.cache);
        if (!headers || headers.length === 0) {
          return {
            // The field is unspecifed or declared explicitly to include no headers,
            // so we need to return 0 quantity.  Default forwarding with no caching.
            Quantity: 0,
            Items: []
          };
        } else if (indexOf.call(headers, "*") >= 0) {
          // Wildcard specificaton.  Everything gets forwarded with no caching.
          if (headers.length === 1) {
            return {
              Quantity: 1,
              Items: ["*"]
            };
          } else {
            throw new Error("Incorrect header specificaton.  Wildcard cannot be used with other named headers.");
          }
        } else {
          return {
            // Named, finite headers specified.  These get forwarded AND cached by CloudFront.
            Quantity: headers.length,
            Items: headers
          };
        }
      };
      buildOrigins = function(name, originID) {
        return {
          Quantity: 1,
          Items: [
            {
              Id: originID,
              DomainName: buildSource(name),
              CustomHeaders: {
                Quantity: 0,
                Items: []
              },
              OriginPath: "",
              CustomOriginConfig: {
                HTTPPort: 80,
                HTTPSPort: 443,
                OriginProtocolPolicy: "http-only",
                OriginSslProtocols: {
                  Quantity: 1,
                  Items: ["TLSv1.2"]
                },
                OriginReadTimeout: 30,
                OriginKeepaliveTimeout: 5
              }
            }
          ]
        };
      };
      // This method takes into consideration the input configuration and sets smart defaults for CloudFront configuration
      // to fill the gaps where the user has no opinon set.
      applyDefaults = async(function*(name) {
        var httpVersion, priceClass, ssl;
        ({ssl, priceClass, httpVersion} = config.aws.cache);
        return {
          protocolPolicy: ssl ? "redirect-to-https" : "allow-all",
          priceClass: priceClass || "100",
          originID: "Haiku9-" + regularlyQualify(name),
          cert: (yield setViewerCertificate()),
          headers: setHeaderCacheConfiguration(),
          expires: config.aws.cache.expires || 60,
          httpVersion: httpVersion || "http2"
        };
      });
      // This helper constructs a CloudFront distribution configuration.  It optionally
      // accepts a pre-existing configuration to faciliate a deepEqual comparison for
      // update detection.
      buildConfiguration = async(function*(name, c = {}) {
        var distro;
        distro = (yield applyDefaults(name));
        // Fill out configuration for CloudFront distribution... it's a doozy.
        c.CallerReference = c.CallerReference || "Haiku " + randomKey(32);
        c.Comment = "Origin is S3 bucket. Setup by Haiku9.";
        c.Enabled = true;
        c.PriceClass = "PriceClass_" + distro.priceClass;
        c.ViewerCertificate = distro.cert;
        c.HttpVersion = distro.httpVersion;
        c.DefaultRootObject = "";
        c.Aliases = {
          Quantity: 1,
          Items: [name]
        };
        c.Origins = c.Origins || buildOrigins(name, distro.originID);
        c.DefaultCacheBehavior = {
          TargetOriginId: distro.originID,
          SmoothStreaming: false,
          MinTTL: 0,
          MaxTTL: distro.expires,
          DefaultTTL: distro.expires,
          ViewerProtocolPolicy: distro.protocolPolicy,
          Compress: false,
          ForwardedValues: {
            Cookies: {
              Forward: "all"
            },
            Headers: setHeaderCacheConfiguration(),
            QueryString: true,
            QueryStringCacheKeys: {
              Quantity: 1,
              Items: ["*"]
            }
          },
          TrustedSigners: {
            Enabled: false,
            Quantity: 0,
            Items: []
          },
          AllowedMethods: {
            Items: ["GET", "HEAD", "OPTIONS", "PUT", "PATCH", "POST", "DELETE"],
            Quantity: 7,
            CachedMethods: {
              Items: ["GET", "HEAD", "OPTIONS"],
              Quantity: 3
            }
          },
          LambdaFunctionAssociations: {
            Quantity: 0,
            Items: []
          }
        };
        return c;
      });
      createDistribution = async(function*(name) {
        var distribution, e, params;
        console.log(`-- Creating CloudFront CDN Distribution for ${name}`);
        try {
          params = {
            DistributionConfig: (yield buildConfiguration(name))
          };
          distribution = (yield cf.createDistribution(params));
          distribution.isNew = true;
          return distribution;
        } catch (error) {
          e = error;
          console.error("Unexpected response while creating CloudFront" + "distribution.", e);
          throw new Error();
        }
      });
      updateDistribution = async(function*(ETag, Distribution) {
        var e, params;
        console.log(`-- Updating CloudFront CDN Distribution for ${name}`);
        try {
          params = {
            Id: Distribution.Id,
            IfMatch: ETag,
            DistributionConfig: Distribution.DistributionConfig
          };
          return (yield cf.updateDistribution(params));
        } catch (error) {
          e = error;
          console.error("Unexpected response while updating CloudFront" + "distribution.", e);
          throw new Error();
        }
      });
      // This recursive helper smooths out arrays within nested objects so that we
      // can safely apply a deepEqual to compare current and new configurations.
      deepSort = function(o) {
        var k, n, v;
        if (Array.isArray(o)) {
          return o.sort();
        } else if (typeof o === "object") {
          n = {};
          for (k in o) {
            v = o[k];
            n[k] = deepSort(v);
          }
          return n;
        } else {
          return o;
        }
      };
      // Compare the current configuration we fetched from AWS to our desired end
      // state.  Because the configuration is complex and filled with optional fields,
      // we designate the desired configuration as a transformation on the current.
      // If this causes changes, then we need to issue a time consuming update.
      confirmDistributionConfig = async(function*(name, {ETag, Distribution}) {
        var current, newconfig;
        current = deepSort(Distribution.DistributionConfig);
        newconfig = deepSort((yield buildConfiguration(name, Object.assign({}, current))));
        if (deepEqual(current, newconfig)) {
          return Distribution;
        } else {
          return updateDistribution(ETag, Distribution);
        }
      });
      try {
        // Search the user's current distributions for ones that matches our needs.
        // If we don't find one, create it. If it's misconfigured, update it.
        list = ((yield cf.listDistributions({}))).DistributionList.Items;
        pattern = {
          Aliases: {
            Quantity: 1,
            Items: [regularlyQualify(name)]
          }
        };
        matches = collect(where(pattern, list));
        if (empty(matches)) {
          // Create a distribution for this hostname, if it doesn't exist.
          return createDistribution(name);
        }
        // If the distribution does exist, get its ID and current version ETag.
        match = matches[0];
        id = match.Id;
        current = (yield cf.getDistribution({
          Id: id
        }));
        // Confirm that the distribution is correctly configured. Return it
        // if true, update it if false and return the new object
        return (yield confirmDistributionConfig(name, current));
      } catch (error) {
        e = error;
        console.error("Unexpected response while prepping CloudFront distro", e);
        throw new Error();
      }
    });
    sync = async(function*(distribution) {
      var data, e, results, retry;
      try {
        retry = true;
        results = [];
        while (retry) {
          data = (yield cf.getDistribution({
            Id: distribution.Id
          }));
          if (data.Distribution.Status === "Deployed") {
            results.push(retry = false);
          } else {
            results.push((yield sleep(15000)));
          }
        }
        return results;
      } catch (error) {
        e = error;
        console.error("Unexpected response while checking CloudFront " + "distribution status.");
        throw new Error();
      }
    });
    syncInvalidation = async(function*(distribution, invalidation) {
      var data, e, results, retry;
      if (invalidation && !distribution.isNew) {
        try {
          retry = true;
          results = [];
          while (retry) {
            data = (yield cf.getInvalidation({
              DistributionId: distribution.Id,
              Id: invalidation.Id
            }));
            if (data.Invalidation.Status === "Completed") {
              results.push(retry = false);
            } else {
              results.push((yield sleep(15000)));
            }
          }
          return results;
        } catch (error) {
          e = error;
          console.error("Unexpected response while checking distribution " + "invalidation status.", e);
          throw new Error();
        }
      }
    });
    return {needsInvalidation, invalidate, set, sync, syncInvalidation};
  });

}).call(this);
