// Generated by CoffeeScript 2.0.2
(function() {
  // Set of helper functions and prepartion steps to setting the DNS record on
  // an S3 bucket's static site or CloudFront distribution.
  var async, collect, deepEqual, empty, first, last, rest, where;

  ({async, last, first, rest, collect, where, empty, deepEqual} = require("fairmont"));

  module.exports = function(config, route53) {
    var bucketURL, build, fullyQualify, regularlyQualify, root;
    ({bucketURL, fullyQualify, regularlyQualify, root} = require("../url")(config));
    // Prepare a set of DNS changes that Route53 uses to present the bucket or
    // CloudFront distribution at the desired URL.  Or, if none, return null.
    build = async(function*(distributions) {
      var addUpsert, changeList, distro, e, getHostedZoneID, hostnames, i, id, j, k, l, len, len1, m, name, reconcile, records, ref, ref1, ref2, sources;
      changeList = []; // Ultimate goal is to fill this with actions.
      
      // Adds a creation event to the DNS batch changes.
      addUpsert = function(source, target) {
        var HostedZoneId;
        if (distributions) {
          HostedZoneId = "Z2FDTNDATAQYW2";
        } else {
          HostedZoneId = require("./s3-hostedzone-ids")[config.aws.region];
        }
        return {
          Action: "UPSERT",
          ResourceRecordSet: {
            Name: target,
            Type: "A",
            AliasTarget: {
              HostedZoneId: HostedZoneId,
              DNSName: fullyQualify(source),
              EvaluateTargetHealth: false
            }
          }
        };
      };
      // Determine if the AWS user owns the requested URL as a public hosted zone
      getHostedZoneID = async(function*() {
        var result, zone, zones;
        zone = root(config.aws.hostnames[0]);
        zones = (yield route53.listHostedZones({}));
        result = collect(where({
          Name: zone
        }, zones.HostedZones));
        if (empty(result)) {
          console.error("It appears you do not have a public hostedzone setup " + `for ${root(config.aws.hostnames[0])}  Without it, H9 cannot setup the DNS ` + "records to route traffic to your bucket.  Aborting.");
          throw new Error();
        }
        return result[0].Id;
      });
      // Scan this hosted zone and determine the changes for this particular record
      reconcile = function(records, hostname, source) {
        var current, desired, record;
        record = collect(where({
          Name: hostname
        }, records));
        if (!empty(record)) {
          // Escape if there's nothing to change.
          desired = addUpsert(source, hostname).ResourceRecordSet;
          current = record[0];
          if (empty(current.ResourceRecords)) {
            delete current.ResourceRecords;
          }
          if (deepEqual(desired, current)) {
            return null;
          }
        }
        // Add an update to the DNS changeList.
        return changeList.push(addUpsert(source, hostname));
      };
      try {
        // "Main" section of build(). Construct the DNS changeList, which is declared
        // above, with reconcile(). Escape if there's nothing to change.
        id = (yield getHostedZoneID());
        records = (yield route53.listResourceRecordSets({
          HostedZoneId: id
        }));
        records = records.ResourceRecordSets;
        hostnames = [];
        sources = [];
        ref = config.aws.hostnames;
        for (j = 0, len = ref.length; j < len; j++) {
          name = ref[j];
          hostnames.push(fullyQualify(name));
        }
        if (distributions) {
          for (k = 0, len1 = distributions.length; k < len1; k++) {
            distro = distributions[k];
            sources.push(distro.DomainName);
          }
        } else {
          for (i = l = 0, ref1 = hostnames.length; 0 <= ref1 ? l < ref1 : l > ref1; i = 0 <= ref1 ? ++l : --l) {
            sources.push(bucketURL());
          }
        }
        for (i = m = 0, ref2 = hostnames.length; 0 <= ref2 ? m < ref2 : m > ref2; i = 0 <= ref2 ? ++m : --m) {
          reconcile(records, hostnames[i], sources[i]);
        }
        if (empty(changeList)) {
          return false;
        } else {
          return {
            HostedZoneId: id,
            ChangeBatch: {
              Changes: changeList
            }
          };
        }
      } catch (error) {
        e = error;
        console.error("Unexpected response while setting DNS records", e);
        throw new Error();
      }
    });
    return {build};
  };

}).call(this);
