// Generated by CoffeeScript 2.0.2
(function() {
  var async, isArray, sleep;

  ({async, sleep, isArray} = require("fairmont"));

  module.exports = function(s3) {
    var buildRule, establish, list, setACL, setCORS;
    establish = async(function*(name, cors) {
      var e, exists;
      try {
        exists = (yield s3.headBucket({
          Bucket: name
        }));
      } catch (error) {
        e = error;
        switch (e.statusCode) {
          case 301:
            console.error("The bucket is in a different region than the client " + "is currently configured to target. Correct the region in your " + ".h9 file.");
            throw new Error();
          case 403:
            console.error("You are not authorized to modify this bucket.");
            throw e;
          case 404:
            exists = false;
            break;
          default:
            console.error("Unexpected reply from AWS", e);
            throw e;
        }
      }
      // If the bucket already exists: ensure it is properly configured.
      if (exists) {
        yield setACL(name);
        if (cors) {
          yield setCORS(name, cors);
        }
        return true;
      }
      try {
        // If the bucket does not exist: create a new, empty S3 bucket.
        // There is a grace period to wait for bucket to be available to API.
        // TODO: Fix by using CloudFormation to do setup instead.
        yield s3.createBucket({
          Bucket: name,
          ACL: "public-read"
        });
        yield sleep(15000);
        if (cors) {
          yield setCORS(name, cors);
        }
        return false;
      } catch (error) {
        e = error;
        console.error("Failed to establish bucket.", e);
        throw new Error();
      }
    });
    // Recursive method to grab all of the object headers in an S3 bucket
    list = async(function*(name, objects, marker) {
      var catTable, data, e, params;
      // Helper to extract data from object header list and creates a lookup table.
      catTable = function(table, data) {
        var i, len, obj;
        for (i = 0, len = data.length; i < len; i++) {
          obj = data[i];
          table[obj.Key] = {
            hash: obj.ETag.split("\"")[1],
            size: obj.Size
          };
        }
        return table;
      };
      try {
        params = {
          Bucket: name,
          Delimiter: '#',
          MaxKeys: 1000
        };
        if (marker) {
          params["Marker"] = marker;
        }
        data = (yield s3.listObjects(params));
        if (data.IsTruncated) {
          objects = catTable(objects, data.Contents);
          return (yield list(name, objects, data.NextMarker));
        } else {
          return catTable(objects, data.Contents);
        }
      } catch (error) {
        e = error;
        console.error("Unexpected reply while pulling S3 bucket keys.", e);
        throw new Error();
      }
    });
    // Set the access control permissions on the whole bucket.
    setACL = async(function*(name) {
      var e;
      try {
        return (yield s3.putBucketAcl({
          Bucket: name,
          ACL: "public-read"
        }));
      } catch (error) {
        e = error;
        console.error("Unexpected response while setting bucket permissions.", e);
        throw new Error();
      }
    });
    buildRule = function(c) {
      var allowedHeaders, allowedMethods, allowedOrigins, exposedHeaders, maxAge, rule;
      ({allowedHeaders = ["*"], allowedMethods = ["GET"], allowedOrigins = ["*"], exposedHeaders = [""]} = c);
      if (c.maxAge != null) {
        maxAge = c.maxAge; // This may be configured to zero for no-cache
      }
      rule = {
        AllowedHeaders: allowedHeaders,
        AllowedMethods: allowedMethods,
        AllowedOrigins: allowedOrigins,
        ExposeHeaders: exposedHeaders
      };
      if (maxAge != null) {
        rule.MaxAgeSeconds = maxAge;
      }
      return rule;
    };
    // Set Cross Origin Resource Sharing (CORS) configuration for the whole bucket.
    setCORS = async(function*(name, config) {
      var c, e, rules;
      if (isArray(config)) {
        rules = (function() {
          var i, len, results;
          results = [];
          for (i = 0, len = config.length; i < len; i++) {
            c = config[i];
            results.push(buildRule(c));
          }
          return results;
        })();
      } else {
        rules = [buildRule(config)];
      }
      try {
        return (yield s3.putBucketCors({
          Bucket: name,
          CORSConfiguration: {
            CORSRules: rules
          },
          ContentMD5: ""
        }));
      } catch (error) {
        e = error;
        console.error("Unexpected response while setting bucket CORS configuration.", e);
        throw new Error();
      }
    });
    return {establish, list, setACL, setCORS};
  };

}).call(this);
