// Generated by CoffeeScript 2.0.2
(function() {
  var async, cat, createReadStream, join, mime, readline, rl, stat;

  readline = require("readline");

  rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout
  });

  ({createReadStream} = require("fs"));

  ({join} = require("path"));

  ({async, cat, stat} = require("fairmont"));

  mime = require("mime");

  // Uploads / Deletes S3 objects as neccessary from the target bucket.
  module.exports = function(config, s3) {
    var bucket;
    bucket = require("./s3")(s3);
    return async(function*({dlist, ulist}) {
      var currentIndex, data, e, file, hash, i, isTooLarge, j, k, len, len1, params, printProgress, results, total, v;
      console.log("-- Syncing S3 bucket.");
      total = (cat(dlist, ulist)).length;
      if (total === 0) {
        rl.close();
        console.log("  ***Warning: S3 Bucket is up-to-date.  Nothing to sync.");
      }
      currentIndex = 0;
      printProgress = function() {
        var progress;
        progress = 100 * currentIndex / total;
        if (progress < 1) {
          progress = progress.toPrecision(2);
        } else if (progress < 10) {
          progress = progress.toPrecision(3);
        } else {
          progress = progress.toPrecision(4);
        }
        rl.write(null, {
          ctrl: true,
          name: 'u'
        });
        rl.write(progress + "%");
        currentIndex++;
        if (currentIndex === total) {
          rl.write(null, {
            ctrl: true,
            name: 'u'
          });
          rl.write("100%");
          rl.write("\n");
          return rl.close();
        }
      };
      // Helper to screen out files larger than 5 GB. They cannot be stored in a
      // single S3 object, which interferes with our static-site ability.
      isTooLarge = async(function*(path) {
        var size;
        ({size} = (yield stat(path)));
        if (size > 4999999999) {
          console.warn(`The file ${path} is larger than 5GB and is too large to store within a single S3 object.  Haiku9 currently does not support multi-object files, so it is skipping this file during the sync.`);
          return true;
        } else {
          return false;
        }
      });
      try {
        // Delete Files
        for (i = 0, len = dlist.length; i < len; i++) {
          file = dlist[i];
          params = {
            Bucket: config.aws.hostnames[0],
            Key: file
          };
          yield s3.deleteObject(params);
          printProgress();
        }
      } catch (error) {
        e = error;
        console.error("Failed to delete object.", e);
        throw new Error();
      }
      try {
        // Cleanup any S3 pseudo "directories" emptied by the deletion.
        data = (yield bucket.list(config.aws.hostnames[0], {}));
        for (k in data) {
          v = data[k];
          if (!(k.match(/.*\/$/ && v.size === 0))) {
            continue;
          }
          params = {
            Bucket: config.aws.hostnames[0],
            Key: k
          };
          yield s3.deleteObject(params);
        }
      } catch (error) {
        e = error;
        console.error("Failed to delete object.", e);
        throw new Error();
      }
      try {
        results = [];
        // Upload Files
        for (j = 0, len1 = ulist.length; j < len1; j++) {
          ({file, hash} = ulist[j]);
          if ((yield isTooLarge(join(config.target, file)))) {
            continue;
          }
          params = {
            Bucket: config.aws.hostnames[0],
            Key: file,
            ACL: "public-read",
            ContentType: mime.lookup(file),
            ContentMD5: new Buffer(hash, "hex").toString('base64'),
            Body: createReadStream(join(config.target, file))
          };
          yield s3.putObject(params);
          if (file.indexOf(".html") > -1) {
            // For HTML files, also publish a copy that lacks the file extension.
            params = {
              Bucket: config.aws.hostnames[0],
              Key: file.split(".html")[0],
              ACL: "public-read",
              ContentType: mime.lookup(file),
              ContentMD5: new Buffer(hash, "hex").toString('base64'),
              Body: createReadStream(join(config.target, file))
            };
            yield s3.putObject(params);
          }
          results.push(printProgress());
        }
        return results;
      } catch (error) {
        e = error;
        console.error("Failed to upload object.", e);
        throw new Error();
      }
    });
  };

}).call(this);
