// Generated by CoffeeScript 2.0.2
(function() {
  // Helper functions to assist with url manipulation for AWS calls.
  var last,
    indexOf = [].indexOf;

  ({last} = require("fairmont"));

  module.exports = function(config) {
    var bucketURL, fullyQualify, regularlyQualify, root;
    // Enforces "fully qualified" form of hostnames and domains.  Idompotent.
    fullyQualify = function(name) {
      if (last(name) === ".") {
        return name;
      } else {
        return name + ".";
      }
    };
    // Named somewhat sarcastically.  Enforces "regular" form of hostnames
    // and domains that is more expected when navigating.  Idompotnent.
    regularlyQualify = function(name) {
      if (last(name) === ".") {
        return name.slice(0, -1);
      } else {
        return name;
      }
    };
    // Given an arbitrary URL, return the fully qualified root domain.
    // https://awesome.example.com/test/42#?=What+is+the+answer  =>  example.com.
    root = function(url) {
      var domain, e, terms;
      try {
        // Remove protocol (http, ftp, etc.), if present, and get domain
        domain = url.split('/');
        domain = indexOf.call(url, "://") >= 0 ? domain[2] : domain[0];
        // Remove port number, if present
        domain = domain.split(':')[0];
        // Now grab the root: the top-level-domain, plus the term to the left.
        terms = regularlyQualify(domain).split(".");
        terms = terms.slice(terms.length - 2);
        // Return the fully qualified version of the root
        return fullyQualify(terms.join("."));
      } catch (error) {
        e = error;
        console.error("Failed to parse root url", e);
        throw new Error();
      }
    };
    // From the bucket name and region, we can construct the bucket's URL.
    // We construct the region-specific endpoint. AWS docs make a vague reference
    // to its ability to "reduce data latency", but it's not clear how many ms
    // we are saving in the response time.
    bucketURL = function() {
      return "s3-website-" + config.aws.region + ".amazonaws.com.";
    };
    return {bucketURL, fullyQualify, regularlyQualify, root};
  };

}).call(this);
